/**
 * Logback: the reliable, generic, fast and flexible logging framework.
 * 
 * Copyright (C) 1999-2007, QOS.ch
 * 
 * This library is free software, you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation.
 */
package ch.qos.logback.eclipse.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.eclipse.ui.IMemento;

import ch.qos.logback.classic.spi.LoggingEvent;

/**
 * This class manages the logging events that are recieved by the plugin.
 * 
 * @author S&eacute;bastien Pennec
 */
public class LoggingEventManager {
  
  private static final String TAG_MGR = "manager"; //$NON-NLS-1$
  private static final String TAG_MAXSIZE = "maxSize"; //$NON-NLS-1$
  private static final double REMOVE_RATIO = 0.33;
  
  static LoggingEvent[] NONE = new LoggingEvent[] {};
  
  private static LoggingEventManager manager;
  
  private int maxSize = 20000;
  private int removeQuantity = 14000;
  private List<LoggingEvent> loggingEventList = Collections.synchronizedList(new ArrayList<LoggingEvent>());

  private List<LoggingEventManagerListener> listeners = new ArrayList<LoggingEventManagerListener>();
  
  private LoggingEventManager() {
  }
  
  public static LoggingEventManager getManager() {
    if (manager == null)
      manager = new LoggingEventManager();
    return manager;
  }
  
  public void addLoggingEvent(LoggingEvent event) {
    if (!EventFilter.filter(event)) {
      return;
    }
    loggingEventList.add(event);
    listSizeCheck();
    sendEvent(event);
  }
  
  private void sendEvent(LoggingEvent event) {
    fireLoggingEventAdded(new LoggingEvent[] { event }, NONE);
  }

  public void addLoggingEventManagerListener(LoggingEventManagerListener listener) {
    if (!listeners.contains(listener))
      listeners.add(listener);
  }

  public void removeLoggingEventManagerListener(LoggingEventManagerListener listener) {
    listeners.remove(listener);
  }

  private void fireLoggingEventAdded(LoggingEvent[] itemsAdded, LoggingEvent[] itemsRemoved) {
    LoggingEventManagerEvent event = new LoggingEventManagerEvent(this, itemsAdded, itemsRemoved);
    Iterator<LoggingEventManagerListener> it = listeners.iterator();
    while(it.hasNext()) {
      (it.next()).loggingEventsChanged(event);
    }
  }
  
  private void listSizeCheck() {
    synchronized(loggingEventList) {
      if (loggingEventList.size() > maxSize) {
        List<LoggingEvent> sub = loggingEventList.subList(0, removeQuantity);
        sub.clear();
      }
    }
  }
  
  public void clearEventList() {
    loggingEventList.clear();
  }
  
  public LoggingEvent getEvent(int index) {
    synchronized(loggingEventList) {
      if (index > loggingEventList.size()-1) {
        return null;
      }
      return loggingEventList.get(index);
    }
  }
  
  public int getEventCount() {
    return loggingEventList.size();
  }
  
  public int getIndex(LoggingEvent event) {
    return loggingEventList.indexOf(event);
  }

  public int getMaxSize() {
    return maxSize;
  }

  public void setMaxSize(int maxSize) {
    this.maxSize = maxSize;
    Double dbl = maxSize * REMOVE_RATIO;
    this.removeQuantity = dbl.intValue();
    listSizeCheck();
  }

  public void saveState(IMemento memento) {
    IMemento mem = memento.createChild(TAG_MGR);
    mem.putInteger(TAG_MAXSIZE, maxSize);
  }
  
  public void init(IMemento memento) {
    IMemento mem = memento.getChild(TAG_MGR);
    if (mem == null) {
      return;
    }
    
    Integer max = mem.getInteger(TAG_MAXSIZE);
    if (max != null) {
      this.maxSize = max;
    }
  }
}
