/**
 * Logback: the reliable, generic, fast and flexible logging framework.
 * 
 * Copyright (C) 1999-2007, QOS.ch
 * 
 * This library is free software, you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation.
 */
package ch.qos.logback.eclipse.views;

import java.net.URL;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ITableColorProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IMemento;

import ch.qos.logback.classic.Level;
import ch.qos.logback.classic.LoggerContext;
import ch.qos.logback.classic.PatternLayout;
import ch.qos.logback.classic.spi.LoggingEvent;
import ch.qos.logback.eclipse.LogbackPlugin;
import ch.qos.logback.eclipse.Messages;
import ch.qos.logback.eclipse.model.LoggingEventManager;

/**
 * This class creates the label that will appear in the logback window to
 * display the content of a LoggingEvent.
 * 
 * @author S&eacute;bastien Pennec
 */
class LoggingEventLabelProvider extends LabelProvider implements ITableLabelProvider,
    ITableColorProvider {

  private static final String TAG_LABEL = "labelProvider"; //$NON-NLS-1$
  private static final String TAG_PATTERN = "pattern"; //$NON-NLS-1$

  private static String DEFAULT_PATTERN = "%relative %level [%thread] %logger{25} %message %nopex"; //$NON-NLS-1$
  private LoggerContext context;
  private PatternLayout patternLayout;

  private Color cachedRed;
  private Color cachedOrange;
  private Color cachedBlack;
  private Color cachedWhite;
  private Color cachedGray;

  private Image stackImage;
  private Image blankImage;

  public LoggingEventLabelProvider() {
    context = new LoggerContext();
    context.setName(Messages.LoggingEventLabelContext);
    createDefaultPatternLayout();
    Display display = Display.getCurrent();
    cachedRed = Display.getCurrent().getSystemColor(SWT.COLOR_RED);
    cachedOrange = new Color(display, 255, 140, 0);
    cachedBlack = Display.getCurrent().getSystemColor(SWT.COLOR_BLACK);
    cachedWhite = Display.getCurrent().getSystemColor(SWT.COLOR_WHITE);
    cachedGray = new Color(display, 240, 240, 240);
    stackImage = loadImage("/icons/call_hierarchy.gif"); //$NON-NLS-1$
    blankImage = loadImage("/icons/blank.gif"); //$NON-NLS-1$
  }

  private Image loadImage(String path) {
    URL url = LogbackPlugin.getDefault().getBundle().getEntry(path);
    ImageDescriptor descriptor = ImageDescriptor.createFromURL(url);
    return descriptor.createImage();
  }

  public String getPattern() {
    return patternLayout.getPattern();
  }

  public void updatePattern(String pattern) {
    patternLayout.stop();
    patternLayout.setPattern(pattern);
    patternLayout.start();
  }

  public void saveState(IMemento memento) {
    IMemento mem = memento.createChild(TAG_LABEL);
    mem.putString(TAG_PATTERN, patternLayout.getPattern());
  }

  public void init(IMemento memento) {
    IMemento mem = memento.getChild(TAG_LABEL);
    if (mem == null) {
      createDefaultPatternLayout();
      return;
    }

    String pattern = mem.getString(TAG_PATTERN);
    if (pattern == null || pattern.length() == 0) {
      createDefaultPatternLayout();
      return;
    }

    patternLayout = new PatternLayout();
    patternLayout.setContext(context);
    patternLayout.setPattern(pattern);
    patternLayout.start();
  }

  private void createDefaultPatternLayout() {
    patternLayout = new PatternLayout();
    patternLayout.setContext(context);
    patternLayout.setPattern(DEFAULT_PATTERN);
    patternLayout.start();
  }

  public Image getColumnImage(Object element, int columnIndex) {
    LoggingEvent event = (LoggingEvent) element;

    if (event.getThrowableProxy() != null) {
      return stackImage;
    }

    return blankImage;
  }

  public String getColumnText(Object element, int columnIndex) {
    if (!(element instanceof LoggingEvent)) {
      return ""; //$NON-NLS-1$
    }

    LoggingEvent event = (LoggingEvent) element;

    return patternLayout.doLayout(event);
  }

  public Color getBackground(Object element, int columnIndex) {
    LoggingEvent event = (LoggingEvent) element;
    int index = LoggingEventManager.getManager().getIndex(event);
    if (index % 2 == 0) {
      return cachedGray;
    } else {
      return cachedWhite;
    }
  }

  public Color getForeground(Object element, int columnIndex) {
    LoggingEvent event = (LoggingEvent) element;
    if (event.getLevel().equals(Level.ERROR)) {
      return cachedRed;
    } else if (event.getLevel().equals(Level.WARN)) {
      return cachedOrange;
    } else {
      return cachedBlack;
    }
  }

  @Override
  public void dispose() {
    super.dispose();
    stackImage.dispose();
    cachedGray.dispose();
    cachedOrange.dispose();
  }
}